@@ -20,6 +20,7 @@
 #include "qpython.h"
 #include "qpython_imageprovider.h"
 #include "global_libpython_loader.h"
+#include "pythonlib_loader.h"
 
 #include "pyotherside_plugin.h"
 
@@ -50,6 +51,9 @@ PyOtherSideExtensionPlugin::initializeEngine(QQmlEngine *engine, const char *uri
     // load libpython again RTLD_GLOBAL again. We do this here.
     GlobalLibPythonLoader::loadPythonGlobally();
 
+    // Extract and load embedded Python Standard Library, if necessary
+    PythonLibLoader::extractPythonLibrary();
+
     engine->addImageProvider(PYOTHERSIDE_IMAGEPROVIDER_ID, new QPythonImageProvider);
 }
 
@@ -61,4 +65,5 @@ PyOtherSideExtensionPlugin::registerTypes(const char *uri)
     // There is no PyOtherSide 1.1 import, as it's the same as 1.0
     qmlRegisterType<QPython12>(uri, 1, 2, PYOTHERSIDE_QPYTHON_NAME);
     qmlRegisterType<QPython13>(uri, 1, 3, PYOTHERSIDE_QPYTHON_NAME);
+    qmlRegisterType<QPython14>(uri, 1, 4, PYOTHERSIDE_QPYTHON_NAME);
 }
