@@ -19,6 +19,7 @@
 #include "qpython_priv.h"
 
 #include "qpython_imageprovider.h"
+#include "ensure_gil_state.h"
 
 #include <QDebug>
 
@@ -36,9 +37,9 @@ static void
 cleanup_python_qimage(void *data)
 {
     QPythonPriv *priv = QPythonPriv::instance();
-    priv->enter();
+
+    ENSURE_GIL_STATE;
     Py_XDECREF(static_cast<PyObject *>(data));
-    priv->leave();
 }
 
 QImage
@@ -83,20 +84,20 @@ QPythonImageProvider::requestImage(const QString &id, QSize *size, const QSize &
     //
     // pyotherside.set_image_provider(image_provider)
 
-    priv->enter();
-    PyObject *args = Py_BuildValue("(N(ii))",
-            PyUnicode_FromString(id_utf8.constData()),
-            requestedSize.width(), requestedSize.height());
-    PyObject *result = PyObject_Call(priv->image_provider, args, NULL);
-    Py_DECREF(args);
+    ENSURE_GIL_STATE;
 
-    if (result == NULL) {
+    PyObjectRef args(Py_BuildValue("(N(ii))",
+            PyUnicode_FromString(id_utf8.constData()),
+            requestedSize.width(), requestedSize.height()), true);
+    PyObjectRef result(PyObject_Call(priv->image_provider.borrow(), args.borrow(), NULL), true);
+
+    if (!result) {
         qDebug() << "Error while calling the image provider";
         PyErr_Print();
         goto cleanup;
     }
 
-    if (!PyArg_ParseTuple(result, "O(ii)i", &pixels, &width, &height, &format)) {
+    if (!PyArg_ParseTuple(result.borrow(), "O(ii)i", &pixels, &width, &height, &format)) {
         PyErr_Clear();
         qDebug() << "Image provider must return (pixels, (width, height), format)";
         goto cleanup;
@@ -179,8 +180,6 @@ QPythonImageProvider::requestImage(const QString &id, QSize *size, const QSize &
     }
 
 cleanup:
-    Py_XDECREF(result);
-    priv->leave();
 
     *size = img.size();
     return img;
