@@ -32,7 +32,7 @@ QPythonWorker::~QPythonWorker()
 }
 
 void
-QPythonWorker::process(QString func, QVariant args, QJSValue *callback)
+QPythonWorker::process(QVariant func, QVariant args, QJSValue *callback)
 {
     QVariant result = qpython->call_sync(func, args);
     if (callback) {
