@@ -34,6 +34,9 @@ class TestPyOtherSide : public QObject {
         void testEvaluate();
         void testQVariantConverter();
         void testPyObjectConverter();
+        void testPyObjectRefRoundTrip();
+        void testPyObjectRefAssignment();
+        void testQObjectRef();
         void testConvertToPythonAndBack();
         void testSetToList();
 };
